import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import _ from 'lodash';
import format from 'string-template';

import logging from '../logging';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  entity?: string;
  query?: string;
}

interface IUri {
  uri?: string;
}

interface IResult {
  payload: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData & IUri = task.variables.getAll();

  let retries = task.retries;

  if (_.isNull(retries)) {
    retries = 10;
  }

  try {
    let backendUrlTemplate: string =
      variables.BACKEND_URL + '/{entity}?{query}';

    if (!_.isEmpty(variables.uri)) {
      backendUrlTemplate = variables.BACKEND_URL + '{uri}';
    }

    const entity = await axios({
      method: 'get',
      url: format(backendUrlTemplate, {
        entity: variables.entity,
        query: encodeURI(variables.query),
        uri: encodeURI(variables.uri)
      }),
      headers: { 'X-Username': 'microservice [GET PARAMS]' }
    }).then(response => response.data);

    if (!entity) throw new Error('Entity not found');

    const result: IResult = {
      payload: {
        value: JSON.stringify(entity),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message,
      retries: retries - 1,
      retryTimeout: 60000,
    });
  }
};
