import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import format from 'string-template';
import _ from 'lodash';

import logging from '../logging';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  entity: string;
  id?: string;
}

interface IResult {
  payload: object;
  error: object;
}

interface IError {
  ok: boolean;
  status: number;
  statusText: string;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  let retries = task.retries;

  if (_.isNull(retries)) {
    retries = 10;
  }

  try {
    const backendUrlTemplate: string = variables.BACKEND_URL + '/{entity}/{id}';

    const error = {} as IError;

    const entity = await axios({
      method: 'get',
      url: format(backendUrlTemplate, {
        entity: variables.entity,
        id: variables.id
      }),
      headers: { 'X-Username': 'microservice [GET]' }
    })
      .then(response => {
        error.ok = true;
        error.status = response.status;
        error.statusText = response.statusText;

        return response.data;
      })
      .catch(err => {
        if (err.response.status === 404) {
          error.ok = false;
          error.status = err.response.status;
          error.statusText = err.response.statusText;

          return {};
        }

        throw err;
      });

    const result: IResult = {
      payload: {
        value: JSON.stringify(entity),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      error: {
        value: JSON.stringify(error),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message,
      retries: retries - 1,
      retryTimeout: 60000,
    });
  }
};
