/* tslint:disable no-null-keyword */

import amqp from 'amqp-connection-manager';
import _ from 'lodash';

import logging from '../logging';
import debug from '../debug';

interface IConfig {
  RABBITMQ_URL: string;
}

interface IData {
  exchange?: string;
  priority: number;
  routing?: string;
  queue?: string;
  type: string;
  payload: string;
}

let amqpConnection = null;
let amqpChannels = {};

// const start = async (RABBITMQ_URL) => {
//   amqpConn = await amqp.connect(RABBITMQ_URL + '?heartbeat=60', (err, conn) => {
//     console.log(111111111111111111111111, RABBITMQ_URL);

//     if (err) {
//       console.error('[AMQP]', err.message);

//       return setTimeout(start, 2000, RABBITMQ_URL);
//     }

//     conn.on('error', function(err) {
//       if (err.message !== 'Connection closing') {

//         console.error('[AMQP] conn error', err.message);
//       }
//     });

//     conn.on('close', function() {
//       console.error('[AMQP] reconnecting');

//       return setTimeout(start, 2000, RABBITMQ_URL);
//     });

//     console.log('[AMQP] connected');
//   });
// };

export default async ({ task, taskService }) => {

  let amqpChannel = null;

  const variables: IConfig & IData = task.variables.getAll();

  try {
    if (_.isEmpty(amqpConnection) || !amqpConnection.isConnected) {
      console.log('Opening Connection');

      amqpChannels = {};

      amqpConnection = await amqp.connect([variables.RABBITMQ_URL], {
        heartbeatIntervalInSeconds: 10,
        reconnectTimeInSeconds: 5,
      });
    }

    // await start(variables.RABBITMQ_URL);
    // console.log(amqpConn);

    const channel = variables.exchange ? variables.exchange : variables.queue;
    const priority = !variables.priority ? 0 : variables.priority;

    if (_.isEmpty(channel)) {
      throw new Error('Empty Channel');
    }

    if (_.isEmpty(amqpChannels[channel])) {
      console.log('Opening Channel');

      amqpChannels[channel] = await amqpConnection.createChannel();
    }

    amqpChannel = amqpChannels[channel];

    const message = {
      type: variables.type,
      payload: JSON.parse(variables.payload)
    };

    debug({
      type: variables.type === 'taskInfo' ? 'TASK-INFO-ROW-' + message.payload['rowNumber'] : variables.type,
      message: 'Sending ' + (variables.exchange ? 'EXCHANGE' : 'QUEUE') + ' payload data to ELK',
      payload: message,
      variables: variables,
      filename: __filename,
    });

    if (variables.queue) {
      amqpChannel.assertQueue(variables.queue, {
        durable: false,
        autoDelete: false,
        exclusive: false,
        maxPriority: 5,
      });

      await amqpChannel.sendToQueue(
        variables.queue,
        Buffer.from(JSON.stringify(message)),
        {
          persistent: true,
          priority: priority,
        }
      );
    }

    if (variables.exchange) {
      amqpChannel.assertExchange(variables.exchange, 'topic', {
        durable: false,
        // autoDelete: true,
        maxPriority: 5,
      });

      await amqpChannel.publish(
        variables.exchange,
        variables.routing,
        Buffer.from(JSON.stringify(message)),
        {
          persistent: true,
          priority: priority,
        }
      );
    }

    await taskService.complete(task);
  } catch (error) {
    await logging({ error, variables, filename: __filename, type: 'RABBITMQ' });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  } finally {
    if (!_.isEmpty(amqpChannel)) {
      // amqpChannel.close();
    }
    if (!_.isEmpty(amqpConnection)) {
      // amqpConnection.close();
    }
  }
};
