import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import format from 'string-template';
import * as R from 'ramda';
import _ from 'lodash';

import logging from '../logging';

interface IConfig {
  ONEC_URL: string;
  ONEC_UNM: string;
  ONEC_PWD: string;
  lastProcessTime: string;
  currentProcessTime: string;
}

interface IData {
  entity: string;
  filter: string;
  orderby: string;
  skipCounter: number;
  isTypeObject?: boolean;
  top?: number;
}

interface IResult {
  payload?: object;
  size: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const top: number = R.defaultTo(100, variables.top);
    const skip: number = R.defaultTo(0, variables.skipCounter) * top;

    const auth = {
      username: variables.ONEC_UNM,
      password: variables.ONEC_PWD,
    };

    const urlTemplate: string =
      variables.ONEC_URL +
      '/odata/standard.odata/' +
      (variables.isTypeObject
        ? '{entity}?$format=json'
        : '{entity}?$format=json&$top={top}&$skip={skip}&$filter={filter}&$orderby={orderby}');

    const currentProcessTime = new Date(variables.currentProcessTime).toISOString().slice(0, -1);

    const entity = await axios({
      method: 'get',
      auth: !_.isEmpty(variables.ONEC_UNM) ? auth : null,
      url: encodeURI(
        format(urlTemplate, {
          entity: variables.entity,
          filter: format(variables.filter, {
            lastProcessTime: variables.lastProcessTime
              ? variables.lastProcessTime
              : currentProcessTime,
            currentProcessTime: currentProcessTime
          }),
          top: top,
          orderby: variables.orderby,
          skip
        })
      )
    })
      .then(response => response.data)
      .catch(async error => {
        await logging({error, variables, filename: __filename});
        if (error.isAxiosError) {
          return false;
        }
        if (error.response.status === 500) {
          return false;
        }
      });

    const resultTypeList: string   = 'java.util.List<java.lang.Object>';
    const resultTypeObject: string = 'java.util.HashMap<java.lang.Object, java.lang.Object>';

    if (!entity) {
      const result: IResult = {
        size: {
          value: 0,
          type: 'integer'
        },
      };

      await taskService.complete(task, new Variables().setAllTyped(result));
    } else {
      const result: IResult = {
        payload: {
          value: JSON.stringify(variables.isTypeObject ? entity : entity.value),
          type: 'Object',
          valueInfo: {
            objectTypeName: variables.isTypeObject
              ? resultTypeObject
              : resultTypeList,
            serializationDataFormat: 'application/json'
          }
        },
        size: {
          value: variables.isTypeObject ? 1 : R.length(entity.value),
          type: 'integer'
        },
      };

      await taskService.complete(task, new Variables().setAllTyped(result));
    }
  } catch (error) {
    const result: IResult = {
      size: {
        value: 0,
        type: 'integer',
      },
    };
    await taskService.complete(task, new Variables().setAllTyped(result));
  }
};
