/* tslint:disable no-null-keyword */

import { Variables } from 'camunda-external-task-client-js';
import _ from 'lodash';
import moment from 'moment';
import constants from '../const/constants';

import logging from '../logging';
import axios from 'axios';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  payload: string;
}

interface IResult {
  rawRequest: object;
  status: object;
  rowNumber: object;
  trolleyNumber: object;
}

const TROLLEY_TIMEOUT = 2;

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const sync = JSON.parse(variables.payload);

    const isTrolley = sync.trolley ? true : false;

    const taskData = sync.task ? sync.task : sync.trolleyTask;
    const trolleyData = sync.trolley ? sync.trolley : sync.taskTrolley;

    const previousTrolleyData = isTrolley ? trolleyData.previous : trolleyData;

    let rowNumber = null;
    let trolleyNumber = null;
    let status = null;

    if (taskData) {
      if (_.isEmpty(taskData.raw)) {
        status = '100 Missing RAW context';
      } else if (taskData.isHarvesting && !trolleyData) {
        status = '120 Missing TROLLEY context';
      } else if (trolleyData && !taskData.isHarvesting) {
        // status = '130 Non-harvesting task';
      }
    } else {
      status = '110 Missing TASK context';
    }

    let post = null;
    let patch = null;

    if (_.isNull(status) && !_.isEmpty(taskData)) {

      const properties = [];

      for (const property of taskData.rawProperties) {
        const mpProperty = _.find(taskData.properties, (o) => {
          const propStage = property['ПриЗавершении'] ? 'finish' : 'start';

          return o.id == property['Параметр_Key']
            && o.stage == propStage;
        });

        if (!_.isEmpty(mpProperty)) {
          const cProperty = _.cloneDeep(property);
          let value = mpProperty.value;

          if (mpProperty.id === constants.rowId) {
            value = taskData.rowNumber;
            rowNumber = taskData.rowNumber;
          }

          if (mpProperty.id === constants.trolleyId) {
            value = _.get(trolleyData, 'trolley', '');
            trolleyNumber = _.get(trolleyData, 'trolley', '');
          }

          cProperty['РезультатВыполнения'] = value;
          cProperty['РегулярноеВыражение'] = mpProperty.regexp;

          properties.push(cProperty);
        }
      }

      const now = moment();

      const payload = {
        'Факт': taskData.isFinished,
        'Исполнитель': taskData.workerId ? taskData.workerId : '',
        'Исполнитель_Type': taskData.workerId ? 'StandardODATA.Catalog_Сотрудники' : 'StandardODATA.Undefined',
        'ВидРабот_Key': taskData.sectionId,
        'Date': now.format(),
        'ДатаНачала': null, // now.format(),
        'ДатаВыполнения': null, // taskData.finishAt, // now.format(),
      };

      if (!_.isEmpty(properties)) {
        payload['ПараметрыВыполненияРабот'] = properties;
      }

      if (taskData.startedAt) {
        payload['ДатаНачала'] = taskData.startedAt;
        payload['Date'] = taskData.startedAt;
      }

      if (taskData.finishedAt && taskData.isFinished) {
        payload['ДатаВыполнения'] = taskData.finishedAt;
        // payload['Date'] = taskData.finishedAt;
      }

      let isDaySwitch = false ;

      if (payload['ДатаНачала'] && payload['ДатаВыполнения'] && !_.isEmpty(taskData.intervals)) {
        if (!moment(payload['ДатаНачала']).isSame(payload['ДатаВыполнения'], 'day')) {
          _.each(taskData.intervals, interval => {
            if (interval.startedAt && moment(interval.startedAt).isSame(payload['ДатаВыполнения'], 'day')) {
              payload['ДатаНачала'] = interval.startedAt;
              isDaySwitch = true;

              return false;
            }
          });
        }
      }

      if (trolleyData && taskData.isHarvesting) {
        if (!previousTrolleyData) {
          status = '700 Missing Start Trolley';
        } else if (previousTrolleyData.trolley !== trolleyData.trolley) {
          status = '710 Wrong trolley number';
        } else if (previousTrolleyData.type !== 'start') {
          status = '720 Wrong trolley type';
        }

        if (isTrolley) {
          payload['ДатаВыполнения'] = trolleyData.registeredAt;
          payload['Факт'] = true;
        }

        if (previousTrolleyData &&
          moment(payload['ДатаНачала']).isBefore(
            moment(previousTrolleyData.registeredAt)
            )
          ) {
          payload['ДатаНачала'] = previousTrolleyData.registeredAt;
          payload['Date'] = previousTrolleyData.registeredAt;
        }
      }

      payload['Ref_Key'] = sync.id;

      const mergedPayload = _.merge(taskData.raw, payload);

      delete mergedPayload['Number'];

      if (payload['Факт'] && !payload['ДатаВыполнения']) {
        status = '550 Missing Finished Time';
      } else if (!payload['ДатаНачала']) {
        status = '560 Missing Start Time';
      } else if (!payload['Date']) {
        status = '570 Missing Date';
      } else if (!payload['Исполнитель']) {
        status = '580 Missing Worker';
      } else if (moment(payload['ДатаВыполнения']).isBefore(moment(payload['ДатаНачала']))) {
        status = '530 Wrong Date range order';
      } else if (!isDaySwitch && moment(payload['ДатаВыполнения']).diff(
        moment(payload['ДатаНачала']),
        'minutes'
      ) < TROLLEY_TIMEOUT) {
        status = '520 Too fast task';
      }

      for (const property of properties) {
        if (property['Обязательный'] && !property['РезультатВыполнения']) {
          status = '590 Missing Value';

          break;
        }

        if (property['РегулярноеВыражение']) {
          const r = new RegExp('^(' + property['РегулярноеВыражение'] + ')$');

          if (!r.test(property['РезультатВыполнения'].toString())) {
            status = '600 Regexp match failure';
          }
        }
      }

      if (_.isNull(status)) {
        post = {...mergedPayload};
        patch = {...payload};
      }
    }

    if (_.isNull(status) && (_.isEmpty(post) || _.isEmpty(patch))) {
      status = '540 Missing POST || PATCH content';
    }

    if (!_.isNull(status)) {
      rowNumber = trolleyNumber = null;
    }

    let duplicateCount = 0 ;

    if (_.isNull(status) && (!_.isNull(rowNumber) && !_.isNull(trolleyNumber))) {
      const backendUrl: string = variables.BACKEND_URL
        + '/syncs'
        + '?rowNumber=' + rowNumber
        + '&trolleyNumber=' + trolleyNumber
        + '&createdAt[after]=' + encodeURIComponent(moment(patch['ДатаВыполнения']).startOf('day').format()) // TODO: check by trolley/task date ?
        + '&createdAt[before]=' + encodeURIComponent(moment(patch['ДатаВыполнения']).endOf('day').format())
        + '&not[id]=' + sync.id
        // + '&greenhouseId=' + taskData.greenhouseId
        // + '&sectionId=' + taskData.greenhouseId
        + '&page=1'
        + '&itemsPerPage=1';

      duplicateCount = await axios({
        method: 'get',
        headers: {
          Accept: 'application/json',
          'Content-Type': 'application/json',
          'X-Username': 'microservice [RD SYNC TRANSFORM]',
        },
        url: backendUrl,
      })
        .then(response => {
          return response.data.length;
        });
    }

    if (duplicateCount > 0) {
      status = '740 Duplicate params';
    }

    // console.log(
    //   JSON.stringify({
    //     duplicateCount,
    //     status,
    //     trolleyNumber,
    //     rowNumber,
    //     post,
    //     patch
    //   }, null, 4)
    // );

    if (!_.isNull(status)) {
      rowNumber = trolleyNumber = null;
    }

    const result: IResult = {
      rawRequest: {
        value: JSON.stringify({
          post,
          patch,
        }),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      status: {
        value: status,
        type: 'String',
      },
      rowNumber: {
        value: rowNumber,
        type: 'Integer',
      },
      trolleyNumber: {
        value: trolleyNumber,
        type: 'Integer',
      },
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
