import { Variables } from 'camunda-external-task-client-js';

import logging from '../logging';
import axios from 'axios';
import _ from 'lodash';
import moment from 'moment';
import getUuid from 'uuid-by-string';

interface IConfig {
  BACKEND_URL: string;
  TASKS_AMOUNT?: number;
}

interface IData {
  worker: string;
  sectionId?: string;
}

interface IResult {
  payload: object;
  assigned: object;
}
const DEFAULT_MORNING_PERIOD = 2;
const DEFAULT_DAYTIME_PERIOD = 1;
const GUARDING_TASKS_AMOUNT = 5;
const MAX_CHECK_PRIORITY = 4;
const QUALITY_SECTION_ID = '4de01a89-7cc2-11eb-b8a0-00505696f325';
const MAX_TASK_AT_MOMENT = 1;
const MIDDLE_OF_THE_DAY = { hour: 12, minute: 0, second: 0 };
const MIDDLE_OF_THE_DAY_FOR_CONTROLLER = { hour: 9, minute: 0, second: 0 };

const GUARDING_SECTIONS = [
  '59964691-7cc1-11eb-b8a0-00505696f325', // Мониторинг растений огурца на наличие вредителей и болезней
  '43d98007-7cc1-11eb-b8a0-00505696f325', // Мониторинг растений томата на наличие вредителей и болезней
  '4a253bd7-7cc1-11eb-b8a0-00505696f325', // Расселение макролофуса
  '5303039c-7cc1-11eb-b8a0-00505696f325', // Корм макролофуса
  '69729c19-7cc1-11eb-b8a0-00505696f325', // Расселение энтомофагов (пакетированный)
  '61585ad3-7cc1-11eb-b8a0-00505696f325', // Расселение энтомофагов (россыпь)
  '71a4ba81-7cc1-11eb-b8a0-00505696f325', // Опрыскивание растений против вредителей либо болезней
  '7805ec0b-7cc1-11eb-b8a0-00505696f325', // Натягивание лент
  'c75883e1-7cc1-11eb-b8a0-00505696f325', // Опускание/поднятие лент
];

const isTimeBetween = (startTime, endTime, serverTime) => {
  const start = moment(startTime, 'H:mm').subtract(20, 'minute');

  const end = moment(endTime, 'H:mm');
  const server = moment(serverTime, 'H:mm');
  if (end < start) {
    return (
      (server >= start && server <= moment('23:59:59', 'h:mm:ss')) ||
      (server >= moment('0:00:00', 'h:mm:ss') && server < end)
    );
  }
  return server >= start && server < end;
};

// получение технического окна у определенного вида работы в определенной теплицу
const getTechnicalWindow = async (sectionId, greenhouseId, url) => {
  const technicalWindowsSectionUri: string = url + '/technical_window_sections'
    + '?sectionId=' + sectionId
    + '&properties[]=technicalWindowId'
    + '&itemsPerPage=100'
    + '&page=1';

  const technicalWindowsSections = await axios({
    method: 'get',
    url: encodeURI(technicalWindowsSectionUri),
    headers: {
      Accept: 'application/json',
      'Content-Type': 'application/json',
      'X-Username': 'microservice [TASK ASSIGN TECHNICAL WINDOWS SECTION]',
    }
  }).then(response => response.data);

  let technicalWindowsSectionsFilter = '';

  technicalWindowsSections.forEach((technicalWindowsSection: { technicalWindowId: String; }) => {
    technicalWindowsSectionsFilter += '&id[]=' + technicalWindowsSection.technicalWindowId;
  });

  const technicalWindowsUri: string = url + '/technical_windows'
    + '?greenhouseId=' + greenhouseId
    + technicalWindowsSectionsFilter
    + '&properties[]=start'
    + '&properties[]=finish'
    + '&page=1';

  const technicalWindows = await axios({
    method: 'get',
    url: encodeURI(technicalWindowsUri),
    headers: {
      Accept: 'application/json',
      'Content-Type': 'application/json',
      'X-Username': 'microservice [TASK ASSIGN TECHNICAL WINDOWS]',
    }
  }).then(response => response.data);

  let timeFrom;
  let timeTo;

  if (technicalWindows.length === 0) {
    const freqRatingResponse = await axios({
      method: 'get',
      url: url + '/configurations?id=workingTimeConfig&page=1',
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json',
        'X-Username': 'microservice [TASK ASSIGN CONFIGURATIONS]',
      }
    }).then(response => response.data);

    const { start, finish } = freqRatingResponse[0].config;

    timeFrom = start;
    timeTo = finish;
  } else {
    timeFrom = moment(technicalWindows[0].start).format('HH:mm');
    timeTo = moment(technicalWindows[0].finish).format('HH:mm');
  }
  return { timeFrom, timeTo };
};

// корректное завершение скрипта
const finishProcedure = async (task, taskService) => {
  const result: IResult = {
    payload: {
      value: JSON.stringify([]),
      type: 'Object',
      valueInfo: {
        objectTypeName:
          'java.util.ArrayList<java.lang.Object>',
        serializationDataFormat: 'application/json'
      }
    },
    assigned: {
      value: JSON.stringify([]),
      type: 'Object',
      valueInfo: {
        objectTypeName:
          'java.util.ArrayList<java.lang.Object>',
        serializationDataFormat: 'application/json'
      }
    }
  };

  await taskService.complete(task, new Variables().setAllTyped(result));
};

// Получение кол-ва задач для определенной роли работника с учетом времени запроса до\после обеда
const getTaskAmount = async (configName, variables, middleOfTheDay) => {
  const numberOfTask = await axios({
    method: 'get',
    url: variables.BACKEND_URL + `/configurations?id=${configName}&page=1`,
    headers: {
      Accept: 'application/json',
      'Content-Type': 'application/json',
      'X-Username': 'microservice [TASK ASSIGN CONFIGURATIONS 2]',
    }
  }).then(response => response.data);

  const config = (numberOfTask[0] && numberOfTask[0].config) || {};

  const morningPeriod = Number.isInteger(Number(config.morningPeriod))
    ? Number(config.morningPeriod)
    : DEFAULT_MORNING_PERIOD;

  const dayTimePeriod = Number.isInteger(Number(config.dayTimePeriod))
    ? Number(config.dayTimePeriod)
    : DEFAULT_DAYTIME_PERIOD;

  const tasksAmount = moment() <= moment().set(middleOfTheDay) ? morningPeriod : dayTimePeriod;
  const totalTasksAmount = moment() <= moment().set(middleOfTheDay) ? morningPeriod : morningPeriod + dayTimePeriod;

  return { tasksAmount, totalTasksAmount };
};


const addCheckPriorities = async (task, variables) => {
  const checkPrioritiesUri: string = variables.BACKEND_URL + '/check_priorities'
    + '?workerId=' + task.workerId
    + '&sectionId=' + task.sectionId
    + '&properties[]=workerId'
    + '&properties[]=sectionId'
    + '&properties[]=priority'
    + '&properties[]=lastCheck'
    + '&properties[]=isChecking'
    + '&itemsPerPage=100'
    + '&page=1';

  const checkPriorities = await axios({
    method: 'get',
    url: encodeURI(checkPrioritiesUri),
    headers: {
      Accept: 'application/json',
      'Content-Type': 'application/json',
      'X-Username': 'microservice [TASK ASSIGN 5]',
    }
  }).then(response => response.data);

  return { ...task, checkPriority: checkPriorities[0] || {} };
};

// дозагрузка CheckPriorities каждой задаче
const getTasksWithCheckPriorities = async (tasks, variables) => {
  const p = tasks.map(task => addCheckPriorities(task, variables));

  const results = await Promise.all(p);

  return results;
};

// Получение всех завершенных задач сегодня по всем рейтинга на которые можно назначать работы по КК
const getQualityTasks = async (ratings, worker, url) => {
  const qualityRatings = ratings.filter(({ isQuality }) => isQuality);

  let allowedSections = '';

  qualityRatings.forEach(({ sectionId }) => {
    allowedSections += '&sectionId[]=' + sectionId;
  });

  const getTasksSpreadUri = (itemsPerPage, page) => {
    return url + '/tasks?'
      + '&isFinished=true'
      + '&finishedAt[after]=' + moment().format('YYYY-MM-DD')
      + allowedSections
      + '&greenhouseId=' + worker.greenhouseId
      + '&not[workerId]=' + worker.id
      + '&isQuality=false'
      + '&exists[workerId]=true'
      + '&order[finishedAt]=desc'
      + '&properties[]=id'
      + '&properties[]=workerId'
      + '&properties[]=sectionId'
      + '&properties[]=rating'
      + '&properties[]=finishedAt'
      + '&groups[]=mobile'
      + `&page=${page}`
      + `&itemsPerPage=${itemsPerPage}`;
  };

  const tasksSpreadUri: string = getTasksSpreadUri(0, 1);
  const tasksTotalItems = await axios({
    method: 'get',
    url: encodeURI(tasksSpreadUri),
    headers: {
      'X-Username': 'microservice [TASK ASSIGN 4]',
    }
  }).then(response => response.data['hydra:totalItems']);

  const itemsPerPage = 100;
  const numberPages = Math.ceil(tasksTotalItems / itemsPerPage);

  const requests = _.range(1, numberPages + 1).map((page) => {
    return axios({
      method: 'get',
      url: encodeURI(getTasksSpreadUri(itemsPerPage, page)),
      headers: {
        'X-Username': 'microservice [TASK ASSIGN 4]',
      }
    });
  });

  const resultDatas = await Promise.all(requests);
  const result = resultDatas.map(response => response['data']['hydra:member']);

  return _.flatten(result);
};

// Запрос определенной задачи с полным набором полей
const getQualityTask = async (taskId, url) => {
  const tasksSpreadUri: string = url + '/tasks?'
    + '&id=' + taskId
    + '&properties[]=id'
    + '&properties[]=workerId'
    + '&properties[]=parentId'
    + '&properties[]=name'
    + '&properties[]=orderBy'
    + '&properties[]=raw'
    + '&properties[]=greenhouseId'
    + '&properties[]=properties'
    + '&properties[]=rawProperties'
    + '&properties[]=rowNumber'
    + '&properties[]=rating'
    + '&properties[]=finishedAt'
    + '&groups[]=mobile'
    + '&page=1'
    + '&itemsPerPage=1';

  return axios({
    method: 'get',
    url: encodeURI(tasksSpreadUri),
    headers: {
      Accept: 'application/json',
      'Content-Type': 'application/json',
      'X-Username': 'microservice [TASK ASSIGN 7]',
    }
  }).then(response => response['data'][0]);

};

const getTodayTasksCount = (workerId, greenhouseId, sectionId, url) => {
  const tasksQualityUri: string = url + '/tasks?'
    + '&startAt[after]=' + moment().startOf('day').format('YYYY-MM-DD')
    + '&sectionId=' + sectionId
    + '&workerId=' + workerId
    + '&greenhouseId=' + greenhouseId
    + '&properties[]=id'
    + '&groups[]=mobile'
    + '&page=1'
    + '&itemsPerPage=0';

  return axios({
    method: 'get',
    url: encodeURI(tasksQualityUri),
    headers: {
      'X-Username': 'microservice [TASK ASSIGN 6]',
    }
  }).then(response => response.data['hydra:totalItems']);
};

const getTodayCheckedTasksCount = async (workerId, greenhouseId, sectionId, url) => {
  // получаем все задачи проверяемого сотрудника
  const todayTasksUri: string = url + '/tasks?'
    + '&startAt[after]=' + moment().startOf('day').format('YYYY-MM-DD')
    + '&sectionId=' + sectionId
    + '&workerId=' + workerId
    + '&greenhouseId=' + greenhouseId
    + '&properties[]=id'
    + '&groups[]=mobile'
    + '&page=1'
    + '&itemsPerPage=200';

  const todayTasks = await axios({
    method: 'get',
    url: encodeURI(todayTasksUri),
    headers: {
      Accept: 'application/json',
      'Content-Type': 'application/json',
      'X-Username': 'microservice [TASK ASSIGN 9]',
    }
  }).then(response => response.data);

  // задач нет, дальше проверка не нужна
  if (todayTasks.length === 0) {
    return 0;
  }

  let qualityTask = '';

  todayTasks.forEach(({ id }) => {
    qualityTask += '&qualityTaskId[]=' + id;
  });

  // кол-во назначеных как проверяемая у задач КК
  const tasksQualityUri: string = url + '/tasks?'
    + '&startAt[after]=' + moment().startOf('day').format('YYYY-MM-DD')
    + qualityTask
    + '&properties[]=id'
    + '&groups[]=mobile'
    + '&page=1'
    + '&itemsPerPage=0';

  return axios({
    method: 'get',
    url: encodeURI(tasksQualityUri),
    headers: {
      'X-Username': 'microservice [TASK ASSIGN 10]',
    }
  }).then(response => response.data['hydra:totalItems']);
};

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  let retries = task.retries;

  if (_.isNull(retries)) {
    retries = 10;
  }

  try {
    const worker = JSON.parse(variables.worker);
    const sectionId = variables.sectionId;

    if (!worker.greenhouseId) {
      await finishProcedure(task, taskService);

      return;
    }

    const now = new Date();

    const spreadTill = new Date(worker.spreadTill);

    let taskCount = worker.taskCount;

    const tasksAssigned = [];
    const unassignPromises = [];

    const pausedTasksUri: string = variables.BACKEND_URL + '/tasks?'
      + '&properties[]=id'
      + '&workerId=' + worker.id
      + '&startAt[before]=' + moment().startOf('day').format('YYYY-MM-DD')
      + '&exists[startedAt]=true'
      + '&exists[finishedAt]=true'
      + '&isFinished=false'
      + '&groups[]=mobile'
      + '&page=1'
      + '&itemsPerPage=0'; // should be 10, see task list worker.BPMN


    // если хотябы 1 прошлодневная задач на паузе не распределяем новые
    const isHaveTaskOnPause = await axios({
      method: 'get',
      url: encodeURI(pausedTasksUri),
      headers: {
        'X-Username': 'microservice [TASK ASSIGN 1]',
      }
    }).then(response => response.data['hydra:totalItems'] > 0);

    if (isHaveTaskOnPause) {
      await finishProcedure(task, taskService);

      return;
    }

    await Promise.all(unassignPromises);

    if (now <= spreadTill) {
      const greenhouseSectionsUri = variables.BACKEND_URL + '/greenhouse_sections?'
        + '&greenhouseId=' + worker.greenhouseId
        + '&properties[]=sectionId'
        + (sectionId ? '&sectionId=' + sectionId : '')
        + '&itemsPerPage=100'
        + '&onPause=false';

      const greenhouseSections = await axios({
        method: 'get',
        url: encodeURI(greenhouseSectionsUri),
        headers: {
          Accept: 'application/json',
          'Content-Type': 'application/json',
          'X-Username': 'microservice [TASK ASSIGN 3]',
        }
      }).then(response => response.data);

      let allowedGreenhouseSections = '';

      greenhouseSections.forEach((greenhouseSection: { sectionId: String; }) => {
        allowedGreenhouseSections += '&sectionId[]=' + greenhouseSection.sectionId;
      });

      const ratingsUri: string = variables.BACKEND_URL + '/ratings?'
        + '&workerId=' + worker.id
        + '&rating[gt]=0'
        + allowedGreenhouseSections
        + '&order[rating]=desc'
        + '&order[sectionId]=desc'
        + '&properties[]=sectionId'
        + '&properties[]=isQuality'
        + '&properties[]=ratingInt'
        + '&itemsPerPage=100'
        + '&page=1';

      // все рейтинги у проверяемого пользователя
      const ratings = await axios({
        method: 'get',
        url: encodeURI(ratingsUri),
        headers: {
          Accept: 'application/json',
          'Content-Type': 'application/json',
          'X-Username': 'microservice [TASK ASSIGN 2]',
        }
      }).then(response => response.data);

      let taskSpread = false;

      // кол-во задач по периодам для разных ролей
      const numberOfTask = await getTaskAmount('numberOfTask', variables, MIDDLE_OF_THE_DAY);

      const numberOfTaskForController = await getTaskAmount('numberOfTaskForController', variables, MIDDLE_OF_THE_DAY_FOR_CONTROLLER);

      let tasksAmount = _.isNull(variables.TASKS_AMOUNT) ? numberOfTask.tasksAmount : variables.TASKS_AMOUNT;

      for (let i = 0; i < _.size(ratings); i++) {
        const rating = ratings[i];
        const ratingSectionId = rating.sectionId;

        // Если время запуска скрипта выходит за техническое окно текущего вида работы -> пропускаем
        const technicalSectionId = rating.isQuality ? QUALITY_SECTION_ID : rating.sectionId;

        const { timeFrom, timeTo } = await getTechnicalWindow(technicalSectionId, worker.greenhouseId, variables.BACKEND_URL);
        const currentTime = moment().utc().format('HH:mm');

        if (!isTimeBetween(timeFrom, timeTo, currentTime)) {
          continue;
        }

        if (rating.isQuality) {
          // назначаем Контроллеру по 1 задаче КК за раз условие для оптимизации
          if (taskCount >= MAX_TASK_AT_MOMENT) {
            continue;
          }

          // Ко-во задач Контроллера по КК за текущий день
          const todayTasksQualityCount = await getTodayTasksCount(worker.id, worker.greenhouseId, QUALITY_SECTION_ID, variables.BACKEND_URL);

          // Момент запуск скрипта в утренний период и превышение задач КК в нем
          const isMorning = moment() <= moment().set(MIDDLE_OF_THE_DAY_FOR_CONTROLLER);
          const isMoreMorningTasksLimit = isMorning && todayTasksQualityCount > numberOfTaskForController.tasksAmount;

          // Превышение общего кол-во задач КК за день
          const isMoreTotalTasksLimit = todayTasksQualityCount >= numberOfTaskForController.totalTasksAmount;

          // Проверка на максимальное кол-во задач в период соотвествующий моменту запуска скрипта
          if (isMoreMorningTasksLimit || isMoreTotalTasksLimit) {
            continue;
          }

          // выбираем все сегодняшние задачи по всем рейтингам на КК доступные для контроллера
          const qualityTasks = await getQualityTasks(ratings, worker, variables.BACKEND_URL);

          const uniqTasksByWorkerAndSection = _.uniqWith(qualityTasks, (first, second) => first.workerId === second.workerId && first.sectionId === second.sectionId);

          const tasksWithCheckPriorities = await getTasksWithCheckPriorities(uniqTasksByWorkerAndSection, variables);

          const priorities = _.map(
            tasksWithCheckPriorities,
            t => {
              if (_.isEmpty(t.checkPriority)) {
                // Если отсутсвует приоритет считаем что новичок по данному виду работ
                return { sectionId: t.sectionId, workerId: t.workerId, priority: 4 }
              }
              return t.checkPriority
            }
          );

          const workersWithRating = priorities.map(priority => {
            const rating = _.find(ratings, (rating) => rating.sectionId === priority.sectionId) || { ratingInt: 0 };
            const ratingController = rating.ratingInt;

            return { ...priority, ratingController };
          });

          // Сортируем список уникальных работников из доступных задач по приоритетам с учетом последовательности:
          const sortedWorkers = _.orderBy(workersWithRating, [
            'priority', // Сначала новички->Прочие приоритеты
            'ratingController'], // Рейтинг Контроллера по виду работ
            ['desc', 'desc']);

          for (const currentWorker of sortedWorkers) {
            // назначаем Контроллеру по 1 задаче за раз основное условие
            if (taskCount >= MAX_TASK_AT_MOMENT) {
              break;
            }

            // Проверяем что работника сегодня не проверяли по этому виду работы
            const currentWorkerTasksCount = await getTodayCheckedTasksCount(currentWorker.workerId, worker.greenhouseId, currentWorker.sectionId, variables.BACKEND_URL);
            if (currentWorkerTasksCount > 0) {
              continue;
            }

            // выбираем задачу из общего пула начиная поиск с более "старых" задачи
            const sortedTasks = _.orderBy(qualityTasks, 'finishedAt', 'asc');

            let findedTask = _.find(sortedTasks, task => task.workerId === currentWorker.workerId && task.sectionId === ratingSectionId);
            if(!findedTask) {
	            findedTask = _.find(sortedTasks, task => task.workerId === currentWorker.workerId && task.sectionId === currentWorker.sectionId);
	          }

	          if(!findedTask) {
	            findedTask = _.find(sortedTasks, task => task.workerId === currentWorker.workerId);
	          }

            const checkedWorkerTask = await getQualityTask(findedTask.id, variables.BACKEND_URL);

            const qTask = {
              id: getUuid(
                checkedWorkerTask.id + moment().startOf('day').format(),
              ),
              parentId: checkedWorkerTask.parentId,
              name: 'КК ' + checkedWorkerTask.name,
              description: 'Контроль качества',
              requireFinish: false,
              isFinished: false,
              startAt: moment().startOf('day').format(),
              finishAt: moment().endOf('day').format(),
              requireOnlineStart: true,
              orderBy: checkedWorkerTask.orderBy,
              lockedTill: moment().endOf('day').format(),
              raw: checkedWorkerTask.raw,
              workerId: worker.id,
              rawWorkerId: worker.id,
              worker: '/workers/' + worker.id,
              sectionId: QUALITY_SECTION_ID,
              rawSectionId: QUALITY_SECTION_ID,
              section: '/sections/' + QUALITY_SECTION_ID,
              departmentId: worker.departmentId,
              rawDepartmentId: worker.departmentId,
              department: '/departments/' + worker.departmentId,
              positionId: worker.positionId,
              rawPositionId: worker.positionId,
              position: '/positions/' + worker.positionId,
              greenhouseId: checkedWorkerTask.greenhouseId,
              greenhouse: '/greenhouses/' + checkedWorkerTask.greenhouseId,
              properties: checkedWorkerTask.properties,
              rawProperties: checkedWorkerTask.rawProperties,
              ratingSectionId: ratingSectionId,
              rowAmount: null,
              rowNumber: checkedWorkerTask.rowNumber,
              rating: checkedWorkerTask.rating,
              toSend: false,
              pass: 0,
              toExchange: true,
              exclusive: 0,
              isActive: true,
              isQuality: true,
              qualityTaskId: checkedWorkerTask.id,
            };

            let taskInfo;

            await axios({
              method: 'get',
              url: variables.BACKEND_URL + '/tasks/' + qTask.id + '?properties[]=id&groups[]=mobile',
              headers: {
                Accept: 'application/ld+json',
                'Content-Type': 'application/ld+json',
                'X-Username': 'microservice [CONTROLLER TASK GEN]',
              },
            })
              .then(response => response.status)
              .catch(error => {
                if (error.response.status === 404) {
                  return axios({
                    method: 'post',
                    url: variables.BACKEND_URL + '/tasks',
                    headers: {
                      Accept: 'application/ld+json',
                      'Content-Type': 'application/ld+json',
                      'X-Username': 'microservice [TASK GEN 3]',
                    },
                    data: JSON.stringify(qTask)
                  }).then(response => {
                    console.info('SUCCESS: Controller task created!', response.data.id);

                    taskInfo = response.data;
                  })
                    .catch(error => console.error(
                      'ERROR: controller task creating!',
                      _.get(error, 'response.data[\'hydra:description\']', error.response.statusText)
                    ));
                }

                throw error;
              });

            if (taskInfo) {
              taskCount++;
              taskSpread = true;

              tasksAssigned.push(taskInfo);
            }
          }
        } else {
          if (_.includes(GUARDING_SECTIONS, ratingSectionId)) {
            tasksAmount = GUARDING_TASKS_AMOUNT;
          }

          if (taskCount < tasksAmount) {
            const tasksSpreadUri: string = variables.BACKEND_URL + '/tasks?'
              + 'startAt[before]=' + now.toISOString()
              + '&finishAt[after]=' + now.toISOString()
              + '&rating[lte]=' + rating.ratingInt
              + '&sectionId=' + ratingSectionId
              + '&greenhouseId=' + worker.greenhouseId
              + '&exclusive[]=0'
              + '&exclusive[]=' + worker.exclusive
              + '&isFinished=false'
              + '&exists[workerId]=false'
              + '&exists[rawWorkerId]=false'
              // + '&order[rating]=desc'
              + '&order[startAt]=asc'
              + '&order[orderBy]=asc'
              + '&order[rowNumber]=asc'
              + '&properties[]=id'
              + '&properties[]=inProgress'
              + '&groups[]=mobile'
              + '&page=1'
              + '&itemsPerPage=30';

            const tasks = await axios({
              method: 'get',
              url: encodeURI(tasksSpreadUri),
              headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json',
                'X-Username': 'microservice [TASK ASSIGN 4]',
              }
            }).then(response => response.data);

            let index = 0;
            let previousRowOrderBy = 0;

            while (!_.isUndefined(tasks[index]) && taskCount < tasksAmount) {
              if (!tasks[index].inProgress) {
                const taskInfo = await axios({
                  method: 'patch',
                  url: variables.BACKEND_URL + '/tasks/' + tasks[index].id,
                  headers: {
                    'Content-Type': 'application/merge-patch+json',
                    'X-Username': 'microservice [TASK ASSIGN 5]',
                  },
                  data: {
                    workerId: worker.id,
                    // toSend: true,
                  }
                }).then(response => response.data);

                if (taskInfo.workerId === worker.id) {
                  /*
                  axios({
                    method: 'post',
                    url: variables.BACKEND_URL + '/task_metrics/' + tasks[index].id,
                    headers: {
                      'Content-Type': 'application/ld+json',
                      'X-Username': 'microservice [CREATE TASK_METRIC]',
                    },
                    data: {
                      type: 'assign',
                      task: `tasks/${taskInfo.id}`,
                      worker: `workers/${taskInfo.workerId}`,
                      registeredAt: moment().format(),
                    }
                  });
                  */

                  taskCount++;
                  worker.totalTodayTasks++;

                  taskSpread = true;
                  previousRowOrderBy = taskInfo.rowOrderBy;

                  tasksAssigned.push(taskInfo);
                }
              }

              index++;
            }
          }
        }

        if (taskSpread) {
          break;
        }
      }
    }

    // TODO: After switching to beta disable this request
    const tasksUri: string = variables.BACKEND_URL + '/tasks?'
      + 'startAt[before]=' + now.toISOString()
      + '&finishAt[after]=' + now.toISOString()
      + '&workerId=' + worker.id
      + '&isFinished=false'
      + '&isActive=true'
      + '&order[startAt]=asc'
      + '&order[orderBy]=asc'
      + '&order[rowNumber]=asc'
      + '&page=1'
      + '&itemsPerPage=10'; // should be 10, see task list worker.BPMN

    const entity = await axios({
      method: 'get',
      url: encodeURI(tasksUri),
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json',
        'X-Username': 'microservice [TASK ASSIGN 6]',
      },
    }).then(response => response.data);

    const result: IResult = {
      payload: {
        value: JSON.stringify(entity),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.ArrayList<java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      assigned: {
        value: JSON.stringify(tasksAssigned),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.ArrayList<java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    console.error(error);

    await logging({ error, variables, filename: __filename });

    await taskService.handleFailure(task, {
      errorMessage: error.message,
      retries: retries - 1,
      retryTimeout: 10000,
    });
  }
};
