/* tslint:disable no-null-keyword */

import { Variables } from 'camunda-external-task-client-js';
import axios from 'axios';

import _ from 'lodash';
import getUuid from 'uuid-by-string';

import constants from '../const/constants';
import logging from '../logging';
import pLimit from 'p-limit';
import moment from 'moment';
import { hasUncaughtExceptionCaptureCallback } from 'process';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  plan: string;
  sectionId: string;
  greenhouseId: string;
}

interface IResult {
  isProcessed: object;
}

const ROWS_AMOUNT = 540;

export default async ({ task, taskService }) => {
  const variables: IData & IConfig = task.variables.getAll();

  try {
    const plan = JSON.parse(variables.plan);
    const sectionId = variables.sectionId;
    const greenhouseId = variables.greenhouseId;

    const backendUrl: string = variables.BACKEND_URL + '/tasks';

    const weeks = [0]; // [0, 1];

    let isProcessed = false;

    const rawTask = _.get(plan, 'planTemplate.template', null);

    if (rawTask) {

      const rowProperty = _.find(rawTask.ПараметрыВыполненияРабот, (o) => {
        return o.Параметр_Key === constants.rowId;
      });

      if (rowProperty && !rawTask.DeletionMark) {
        _.forEach(weeks, async (weekNumber) => {
          const dow = plan.dow + 1; // Moday == 1
          const week = weekNumber > 0 ? moment().locale('ru').add(weekNumber, 'week') : moment().locale('ru');
          const workDay = week.isoWeekday(dow);

          /* MAIN ROUTINE */

          const rowNumber = _.get(plan, 'row.rowNumber', null);
          const startAt = moment(workDay).locale('ru').startOf('day');
          let finishAt = moment(workDay).locale('ru').startOf('day');

          if (plan.isActive) {
            finishAt = moment(workDay).locale('ru').add(1, 'week').endOf('week');
          }

          let rawProperties = rawTask.ПараметрыВыполненияРабот;

          if (!_.isArray(rawTask.ПараметрыВыполненияРабот)) {
            const tmp = [];

            tmp.push(rawProperties);

            rawProperties = tmp;
          }

          const properties = [];

          for (const rawProperty of rawProperties) {
            const propertyId = rawProperty['Параметр_Key'];

            const propertyPromise = _.isEmpty(propertyId)
              ? new Promise(resolve => resolve(null))
              : axios({
                method: 'get',
                url:
                  variables.BACKEND_URL +
                  '/' +
                  'properties' +
                  '/' +
                  propertyId,
                headers: {
                  Accept: 'application/ld+json',
                  'Content-Type': 'application/merge-patch+json',
                  'X-Username': 'microservice [TASK GEN 1]',
                }
              })
                .then(response => response.data)
                .catch(error => {
                  if (error.response.status === 404) {
                    return null;
                  }

                  throw error;
                });

            const parentProperty = await propertyPromise;

            if (_.isNull(parentProperty)) {
              continue;
            }

            const property = {
              id: rawProperty['Параметр_Key'],
              parentId: rawProperty['Параметр_Key'],
              name: parentProperty['name'],
              description: parentProperty['description'],
              regexp: rawProperty.Параметр_Key === constants.rowId ? '' + rowNumber : (!!rawProperty['РегулярноеВыражение'] ? '' + rawProperty['РегулярноеВыражение'] : parentProperty['regexp']),
              type: parentProperty['type'],
              value: '' + rawProperty['РезультатВыполнения'],
              required: rawProperty['Обязательный'],
              stage: rawProperty['ПриЗавершении'] ? 'finish' : 'start'
            };

            properties.push(property);
          }

          const mupTask = {
            id: getUuid(
              _.toString(rowNumber)
              // + '-' + _.toString(dow)
              + '-' + week.format('YYYY')
              + '-' + week.format('W')
              + '-' + sectionId
              + '-' + greenhouseId
              + '-' + plan.pass
            ),
            parentId: rawTask.Ref_Key,
            name: _.get(plan, 'planTemplate.name', rawTask['Number']),
            description: '' + rawTask['Комментарий'],
            requireFinish: false,
            isFinished: rawTask['Факт'],
            startAt: startAt.toISOString(),
            finishAt: finishAt.toISOString(),
            requireOnlineStart: true,
            orderBy: plan.direction ? rowNumber : ROWS_AMOUNT - rowNumber,
            lockedTill: moment().locale('ru').toISOString(),
            raw: rawTask,
            workerId: null,
            rawWorkerId: rawTask['Исполнитель_Type'] === 'StandardODATA.Catalog_Сотрудники'
              ? rawTask['Исполнитель']
              : null,
            sectionId: null,
            section: null,
            rawSectionId: rawTask['ВидРабот_Key'] === '00000000-0000-0000-0000-000000000000'
              ? null
              : rawTask['ВидРабот_Key'],
            departmentId: null,
            department: null,
            rawDepartmentId: rawTask['Подразделение_Key'] === '00000000-0000-0000-0000-000000000000'
              ? null
              : rawTask['Подразделение_Key'],
            positionId: null,
            position: null,
            rawPositionId: rawTask['Исполнитель_Type'] === 'StandardODATA.Catalog_Должности'
              ? rawTask['Исполнитель']
              : null,
            greenhouseId: null,
            greenhouse: null,
            properties: properties || null,
            rawProperties: rawProperties || null,
            rating_section_id: null,
            row_amount: null,
            row_number: null,
            rating: rawTask['РейтингМУП'] || 0,
            toSend: false,
            pass: plan.pass,
            toExchange: false,
            rowAmount: null,
            rowNumber: rowNumber,
            exclusive: plan.exclusive,
            isActive: plan.isActive,
          };

          mupTask.workerId = null;

          if (mupTask.rawSectionId) {
            mupTask.section = '/sections/' + sectionId;
          }

          if (mupTask.rawDepartmentId) {
            mupTask.department = '/departments/' + mupTask.rawDepartmentId;
          }

          if (mupTask.rawPositionId) {
            mupTask.position = '/positions/' + mupTask.rawPositionId;
          }

          if (greenhouseId) {
            mupTask.greenhouse = '/greenhouses/' + greenhouseId;
          }

          mupTask.orderBy = _.toInteger(
            _.toString(1)
            + _.toString(plan.shift) // modifier to keep some space for task replans
            + _.padStart(mupTask.orderBy, 3, '0')
          );

          axios({
            method: 'get',
            url: backendUrl + '/' + mupTask.id + '?properties[]=id&groups[]=section',
            headers: {
              Accept: 'application/ld+json',
              'Content-Type': 'application/ld+json',
              'X-Username': 'microservice [TASK GEN 2]',
            },
          })
            .then(response => {
              axios({
                method: 'patch',
                // url: backendUrl  + '/' + mupTask.id + '?properties[]=id&groups[]=section',
                url: backendUrl + '/' + mupTask.id,
                headers: {
                  'Content-Type': 'application/merge-patch+json',
                  'X-Username': 'microservice [TASK GEN 4]',
                },
                data: {
                  name: mupTask.name,
                  raw: mupTask.raw,
                  rawProperties: mupTask.rawProperties,
                  properties: mupTask.properties,
                  toExchange: false,
                }
              }).then(response => console.info('SUCCESS: Task patched!', response.data.id))
                .catch(error => console.error(
                  'ERROR: task patching!',
                  _.get(error, 'response.data[\'hydra:description\']', error.response.statusText)
                ));

              return response.status;
            })
            .catch(error => {
              if (error.response.status === 404) {
                axios({
                  method: 'post',
                  url: backendUrl,
                  headers: {
                    Accept: 'application/ld+json',
                    'Content-Type': 'application/ld+json',
                    'X-Username': 'microservice [TASK GEN 3]',
                  },
                  data: JSON.stringify(mupTask)
                }).then(response => console.info('SUCCESS: Task created!', response.data.id))
                  .catch(error => console.error(
                    'ERROR: task creating!',
                    _.get(error, 'response.data[\'hydra:description\']', error.response.statusText)
                  ));

                return error.response.status;
              }

              throw error;
            });

          isProcessed = true;
          /* / MAIN ROUTINE */
        });
      } else {
        console.error('ERROR: Task not created: [ROW MISSING OR TASK DELETED]');
      }

    } else {
      console.error('ERROR: Task not created: [RAW MISSING]');
    }

    const result: IResult = {
      isProcessed: {
        value: isProcessed,
        type: 'Boolean',
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  } finally {
    // do smthn
  }
};
