import { Variables } from "camunda-external-task-client-js";

import * as R from "ramda";

import logging from "../logging";

interface IData {
  taskList: string;
}

interface ITask {
  name: string;
  departmentId: string;
}

type TTaskGroupList = { [name: string]: ITask[] };
type TTaskPair = [string, ITask[]];

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const taskListRaw: ITask | ITask[] = JSON.parse(variables.taskList);
    const taskList = R.is(Array, taskListRaw) ? taskListRaw : [taskListRaw];

    const groupTaskList = (task: ITask): string => task.departmentId;

    const renameProps = (prefix: string) =>
      R.compose<TTaskGroupList, TTaskPair, TTaskGroupList>(
        R.reduce(
          (acc, [key, value]: TTaskPair) =>
            R.set(R.lensProp(prefix + key), value, acc),
          {}
        ),
        R.toPairs
      );

    const transform = R.mapObjIndexed((value: ITask[], key: string) => ({
      key,
      payload: R.map(task => ({ key: "taskInfo", payload: task }), value)
    }));

    const taskListByGroup: { [name: string]: ITask[] } = R.compose(
      R.values,
      R.map(JSON.stringify),
      transform,
      renameProps("department_"),
      R.groupBy(groupTaskList)
    )(taskList);

    await taskService.complete(
      task,
      new Variables().setTyped("collection", {
        value: JSON.stringify(taskListByGroup),
        type: "object",
        valueInfo: {
          objectTypeName: "java.util.ArrayList<java.lang.Object>",
          serializationDataFormat: "application/json"
        }
      })
    );
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
