import amqp from 'amqplib';
import axios from 'axios';
import format from 'string-template';

import redis from '../redis';
import logging from '../logging';

interface IConfig {
  dbUrl: string;
  redisUrl: string;
  rabbitMQUrl: string;
}

interface IDeviceInfo {
  deviceId: string;
  imei: string;
  osVersion: string;
  apiLevel: number;
  device: string;
  model: string;
  product: string;
}

interface IData {
  type: 'taskList';
  workerId: number;
  deviceInfo: string;
  payload: {};
}

interface IWorker {
  personalNumber: number;
}

const getWorkerQueueName = (worker: IWorker, deviceInfo: IDeviceInfo): string =>
  `${worker.personalNumber}_${deviceInfo.imei}`;

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const deviceInfo: IDeviceInfo = JSON.parse(variables.deviceInfo);

    const redisClient = redis({ url: variables.redisUrl });

    const workerUrlTemplate: string = variables.dbUrl + 'workers/{workerId}';
    const worker: IWorker = await axios({
      method: 'get',
      url: format(workerUrlTemplate, { workerId: variables.workerId }),
      headers: { 'X-Username': 'microservice [TASK LIST]' }
    }).then(response => response.data);

    if (!worker) throw new Error('Worker not found');

    const workerRedisKeyTemplate: string = `${worker.personalNumber}_*`;
    const keys: string[] = await redisClient.keys(workerRedisKeyTemplate);

    const connection = await amqp.connect(variables.rabbitMQUrl);

    const channel = await connection.createChannel();

    const workerQueueName: string = getWorkerQueueName(worker, deviceInfo);

    channel.assertQueue(workerQueueName, {
      durable: false
    });

    await Promise.all(
      keys.map(
        async (key: string) =>
          await channel.sendToQueue(
            workerQueueName,
            Buffer.from(await redisClient.get(key))
          )
      )
    );

    await taskService.complete(task);
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
