import axios from 'axios';
import _ from 'lodash';
import qs from 'query-string';

import logging from '../logging';

interface IConfig {
  BACKEND_URL: string;
}

const processWorkers = async (API: string): Promise<boolean> => {
  const params = {
    page: 1
  };

  let process = '/workers?' + qs.stringify(params, { arrayFormat: 'bracket' });

  while (!_.isEmpty(process)) {
    const entity = await axios({
      method: 'get',
      url: API + process,
      headers: {
        accept: 'application/ld+json',
        'X-Username': 'microservice [WRKR TRANSFORM 1]',
      }
    }).then(response => response.data);

    for (const worker of entity['hydra:member']) {
      const historyParams = {
        workerId: worker.id,
        'order[period]': 'desc',
        page: 1
      };

      let search =
        '/position_histories?' +
        qs.stringify(historyParams, { arrayFormat: 'bracket' });

      const histories = await axios({
        method: 'get',
        url: API + search,
        headers: {
          accept: 'application/ld+json',
          'X-Username': 'microservice [WRKR TRANSFORM 2]',
        }
      }).then(response => response.data);

      let positionId = null;
      let departmentId = null;
      let isActive = true;

      for (const history of histories['hydra:member']) {
        if (!history.active) {
          continue;
        }

        if (_.isNull(positionId) && !_.isNull(history.positionId)) {
          positionId = history.positionId;
        }

        if (history.event === 'Увольнение') {
          isActive = false;
        }

        if (_.isNull(departmentId) && !_.isNull(history.departmentId)) {
          departmentId = await axios({
            method: 'get',
            url: API + '/departments/' + history.departmentId,
            headers: {
              Accept: 'application/ld+json',
              'Content-Type': 'application/merge-patch+json',
              'X-Username': 'microservice [WRKR TRANSFORM 3]',
            }
          })
            .then(response => response.data.altId)
            .catch(error => {
              if (error.response.status === 404) {
                return null;
              }

              throw error;
            });
        }

        if (!_.isNull(positionId) && !_.isNull(departmentId)) {
          break;
        }
      }

      const positionIdPromise = _.isEmpty(positionId)
        ? new Promise(resolve => resolve(null))
        : axios({
            method: 'get',
            url: API + '/positions/' + positionId,
            headers: {
              Accept: 'application/ld+json',
              'Content-Type': 'application/merge-patch+json',
              'X-Username': 'microservice [WRKR TRANSFORM 4]',
            }
          })
            .then(response => response.data.id)
            .catch(error => {
              if (error.response.status === 404) {
                return null;
              }

              throw error;
            });

      const departmentIdPromise = _.isEmpty(departmentId)
        ? new Promise(resolve => resolve(null))
        : axios({
            method: 'get',
            url: API + '/departments/' + departmentId,
            headers: {
              Accept: 'application/ld+json',
              'Content-Type': 'application/merge-patch+json',
              'X-Username': 'microservice [WRKR TRANSFORM 5]',
            }
          })
            .then(response => response.data.id)
            .catch(error => {
              if (error.response.status === 404) {
                return null;
              }

              throw error;
            });

      const workerData = {
        positionId: await positionIdPromise,
        departmentId: await departmentIdPromise,
        isActive: isActive
      };

      await axios({
        method: 'patch',
        url: API + '/workers/' + worker.id,
        data: JSON.stringify(workerData),
        headers: {
          Accept: 'application/ld+json',
          'Content-Type': 'application/merge-patch+json',
          'X-Username': 'microservice [WRKR TRANSFORM 6]',
        }
      })
        .then(() => true)
        .catch(error => {
          console.log('PATCH /workers');
          throw error;
        });

      console.log(
        `WORKER [${worker.id}]` +
          ` POSITION: ` +
          workerData.positionId +
          ` DEPARTMENT: ` +
          workerData.departmentId
      );
    }

    process = entity['hydra:view']['hydra:next'];
  }

  return false;
};

export default async ({ task, taskService }) => {
  const variables: IConfig = task.variables.getAll();

  try {
    await processWorkers(variables.BACKEND_URL);

    await taskService.complete(task);
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
